#/usr/bin/env python
import sys
sys.path.append("../tools")
import math
import simreport
import numpy as np
import matplotlib.pyplot as plt;
from matplotlib.font_manager import FontProperties
from operator import itemgetter
import plotparams

def numToDist(distnum):
    if distnum == 1:
        return "gauss"
    elif distnum == 2:
        return "poisson"
    elif distnum == 3:
        return "geometric"
    return "binomial"

def distToNum(distname):
    if distname == "gauss":
        return 1
    elif distname == "poisson":
        return 2
    elif distname == "geometric":
        return 3
    return 0

def importdata(filename, numslots, numpart, dist):
    reportlist = simreport.importReportBatch(filename)
    avglist = simreport.averageReports(reportlist)
    distnum = distToNum(dist)
    reportdata = []
    for report in avglist:
        if (report.numslots == numslots and
            report.numpart == numpart and
            report.distribution == distnum):
            reportdata.append(float("%.1f" % (report.time_exec * 1.0 / 1000.0)))
    return (reportdata)

def importhitdata(filename, numslots, numpart, dist):
    reportlist = simreport.importReportBatch(filename)
    avglist = simreport.averageReports(reportlist)
    distnum = distToNum(dist)
    for report in avglist:
        if (report.numslots == numslots and
            report.numpart == numpart and
            report.distribution == distnum):
            return report
    return None

def plotgroup(dist):
    plots = []
    reportdata = []
#    reportdata.append(["Partitions", "Auto", "Noauto"])
    for numpart in [2, 4, 8]:
        data_noauto = importdata("bench_noautopart.bin", 1000, numpart, dist)
        data_auto = importdata("bench_autopart_straight.bin", 1000, numpart, dist)
        assert(len(data_noauto) == len(data_auto))
        for i in range(len(data_auto)):
            result_auto = data_auto[i]
            result_noauto = data_noauto[i]
            reportdata.append([numpart, result_auto, result_noauto])

    N = 3
    ind = np.arange(N) + 0.1  # the x locations for the groups
    width = 0.4        # the width of the bars
    reportdata = []
    for numpart in [2, 4, 8]:
        data = importdata("bench_noautopart.bin", 1000, numpart, dist)
        reportdata.append(data[0])
    plot_noauto = plt.bar(ind, reportdata, width, color='r')
    autolabel(plot_noauto)

    reportdata = []
    for numpart in [2, 4, 8]:
        data = importdata("bench_autopart_straight.bin", 1000, numpart, dist)
        reportdata.append(data[0])
    plot_auto = plt.bar(ind + width, reportdata, width, color='b')
    autolabel(plot_auto)

    plt.legend((plot_noauto[0], plot_auto[0]), ('Fixed', 'Auto'))
    plt.ylabel('latency (ms)')
    plt.ylim(0, 10)
    plt.title(dist)
    plt.xticks(ind+(width), ('2 part', '4 part', '8 part'))

def autolabel(rects):
    # attach some text labels
    for rect in rects:
        height = rect.get_height()
        plt.text(rect.get_x()+rect.get_width()/2., 0.2 + height, '%.1f'%float(height), ha='center', va='bottom')

def plotslots(dist):
    plt.figure()
    N = 3
    ind = np.arange(N) + 0.2  # the x locations for the groups
    width = 0.15        # the width of the bars
    i = 0
    palette = {1 : 'r', 10 : 'g', 100 : 'b', 1000 : 'cyan'}
    plots = []
    legends = []
    for numslot in [1, 10, 100, 1000]:
        reportdata = []
        for numpart in [2, 4, 8]:
            data_auto = importdata("bench_autopart_straight.bin", numslot, numpart, dist)
            reportdata.append(data_auto[0])
        plot = plt.bar(ind + (width * i), reportdata, width, color=palette[numslot])
        autolabel(plot)
        plots.append(plot[0])
        legends.append("slots = " + str(numslot))
        i += 1

    plt.legend(plots, legends)
    plt.ylabel('latency (ms)')
    plt.ylim(0, 10)
    plt.title(dist)
    plt.xticks(ind+(width * 2), ('2 partitions', '4 partitions', '8 partitions'))
    plt.savefig("autopart_slot_" + dist + ".svg", format="svg")

def plothitstat():
    plt.figure()
    plots = []
    N = 4
    report = importhitdata("bench_autopart_straight.bin", 1000, N, "geometric")
    ind = np.arange(N) + 0.1  # the x locations for the groups
    width = 0.4        # the width of the bars
    plots = []
    totalrange = 0
    for part in range(N):
        totalrange += report.hitstat[part]
    reportdata = []
    for part in range(N):
        reportdata.append(1.0 * report.hitstat[part] / totalrange)

    plot = plt.bar(ind, reportdata, width, color='r')
    autolabel(plot)

plotslots("geometric")
plt.figure()
plt.subplot(121)
plotgroup("binomial")
plt.subplot(122)
plotgroup("geometric")
plt.savefig("autopart_1.svg", format="svg")
plt.figure()
plt.subplot(121)
plotgroup("gauss")
plt.subplot(122)
plotgroup("poisson")
plt.savefig("autopart_2.svg", format="svg")
