#/usr/bin/env python
import sys
sys.path.append("../tools")
import math
import simreport_v2
import numpy as np
import matplotlib.pyplot as plt;
from matplotlib.font_manager import FontProperties
from operator import itemgetter
import plotparams

def numToDist(distnum):
    if distnum == 1:
        return "gauss"
    elif distnum == 2:
        return "poisson"
    elif distnum == 3:
        return "geometric"
    return "binomial"

def distToNum(distname):
    if distname == "gauss":
        return 1
    elif distname == "poisson":
        return 2
    elif distname == "geometric":
        return 3
    return 0

def importhitdata(filename, numslots, numpart, dist):
    reportlist = simreport_v2.importReportBatch(filename)
    avglist = simreport_v2.averageReports(reportlist)
    distnum = distToNum(dist)
    for report in avglist:
        if (report.numslots == numslots and
            report.numpart == numpart and
            report.distribution == distnum):
            return report
    return None

def autolabel(rects):
    # attach some text labels
    for rect in rects:
        height = rect.get_height()
        plt.text(rect.get_x()+rect.get_width()/2., 0.001 + height, '%.2f'%float(height),
                fontsize=10, ha='center', va='bottom')

def plothitstat():
    NUMDOCS=40000000
    NUMPART = 8
    NUMSLOTS = 1
    PARTRANGE = NUMDOCS / NUMPART
    SLOTRANGE = NUMDOCS / (NUMPART * NUMSLOTS)

    plt.figure()
    plots = []
    report = importhitdata("bench_autopart.bin", 1000, NUMPART, "gauss")
    plots = []
    totalrange = 0
    numqueries = 0
    for part in range(NUMPART):
        totalrange += report.partitionscheme[part]
        numqueries = report.numqueries
    reportdata = []
    lefts = []
    for part in range(NUMPART):
        lefts.append(PARTRANGE * part)
        reportdata.append(1.0 * report.partitionscheme[part] / totalrange)

    mu, sigma = NUMDOCS / 2, NUMDOCS / 4
    #x = np.random.geometric(0.0000001, numqueries)
    x = np.random.normal(mu, sigma, numqueries)

    # Filter out any docs not within range
    actual = []
    for num in x:
        if num > 0 and num < NUMDOCS:
            actual.append(num)

    # Create a histogram of the data
    (n, bins) = np.histogram(actual, 8 * NUMSLOTS) #, normed=1)
    total = sum(n)
    graph = []
    idealleft = []
    i = 0
    for num in n:
        idealleft.append(i * SLOTRANGE)
        graph.append(1.0 * num / total)
        i += 1
    plot1 = plt.plot(map(lambda x : x + (PARTRANGE / 2), idealleft), graph, color='b', alpha=0.8)
    plot2 = plt.bar(lefts, reportdata, PARTRANGE, color='r', alpha=0.5)
    plt.ylabel("Probability")
    plt.xticks(map(lambda x : x + (PARTRANGE / 2), lefts), ('1', '2', '3', '4', '5', '6', '7', '8'))
    plt.legend((plot1[0], plot2[0]), ('Input distribution', 'Slot distribution (after simulation)'))
    plt.savefig("autopart_mapping_gauss.svg", format="svg")

plothitstat()
