#/usr/bin/env python
import sys
import pylab
import numpy as np
import matplotlib.pyplot as plt;
import plotparams
from operator import itemgetter
import math

NUMDOCS=0
NUMPART=1
NUMCONN=2
HEAPSIZE=3
AFFINITY=4
RATIO=5

class Cacheentry:
    def __init__(self, line):
        tokens = line.split(" ")
        self.numdocs = int(tokens[NUMDOCS])
        self.numpart = int(tokens[NUMPART])
        self.numconn = int(tokens[NUMCONN])
        self.heapsize = int(tokens[HEAPSIZE])
        self.affinity = bool(int(tokens[AFFINITY]))
        self.ratio = [float(tokens[RATIO])]
        self.ratio_mean = 0.0
        self.ratio_stddev = 0.0

    def equal(self, entry):
        return (self.numdocs == entry.numdocs and self.numpart == entry.numpart
            and self.numconn == entry.numconn and self.heapsize == entry.heapsize
            and self.affinity == entry.affinity)

    def add(self, entry):
        self.ratio.extend(entry.ratio)

    def printEntry(self):
        print self.numdocs, self.numpart, self.numconn, self.heapsize, self.affinity, self.ratio_mean

    def average(self, howmany):
        self.ratio_mean = sum(self.ratio) / howmany
        for num in self.ratio:
            self.ratio_stddev += math.pow(num - self.ratio_mean, 2)
        self.ratio_stddev = math.sqrt(self.ratio_stddev)

def importcachedata(filename):
    fh = open(filename, 'r')
    stats = []
    for line in fh:
        stats.append(Cacheentry(line))
    return (stats)

def importAverage(filename):
    reportlist = importcachedata(filename)
    avgstats = []
    avglist = []
    numavg = []
    for report in reportlist:
        found = False
        for i in range(len(avglist)):
            if report.equal(avglist[i]):
                avglist[i].add(report)
                numavg[i] += 1
                found = True
                break
        if not found:
            avglist.append(report)
            numavg.append(1)
    for i in range(len(avglist)):
        avglist[i].average(numavg[i])
    return avglist


def getratios(stats, numpart, numconn, heapsize, affinity):
    ratios = []
    numdocs = []
    ratiodev = []
    for entry in stats:
        if (entry.numpart == numpart and entry.numconn == numconn and entry.affinity == affinity and entry.heapsize == heapsize):
            numdocs.append(entry.numdocs)
            ratios.append(entry.ratio_mean)
            ratiodev.append(entry.ratio_stddev)
    return (numdocs, ratios, ratiodev)
#    return (Gnuplot.PlotItems.Data(sortedratios, with_="linespoints", title=("-p %d -c %d %s" % (numpart, numconn, aff))))

L1LAT = 1.7
L2LAT = 4.2
L3LAT = 16.7
MEMLAT = 100.0
def getaccesstimes(l1stats, l2stats, l3stats, numpart, numconn, heapsize, affinity):
    numdocs = []
    accesstimes = []
    for i in range(min(len(l1stats), len(l2stats), len(l3stats))):
        l1entry = l1stats[i]
        l2entry = l1stats[i]
        l3entry = l1stats[i]
        if (l1entry.numpart == l2entry.numpart == l3entry.numpart == numpart and
            l1entry.numconn == l2entry.numconn == l3entry.numconn == numconn and
            l1entry.affinity == l2entry.affinity == l3entry.affinity == affinity and
            l1entry.numdocs == l2entry.numdocs == l3entry.numdocs and
            l1entry.heapsize == l2entry.heapsize == l3entry.heapsize == heapsize):

            l1miss = l1entry.ratio_mean
            l2miss = l2entry.ratio_mean
            l3miss = l3entry.ratio_mean
            accesstime = (L1LAT + (l1miss * (L2LAT + (l2miss * (L3LAT + (l3miss * MEMLAT))))))
            numdocs.append(l1entry.numdocs)
            accesstimes.append(accesstime)
    return (numdocs, accesstimes)

def plotratios(stats, numpart, numconn, heapsize, affinity):
    (numdocs, ratios, ratiodev) = getratios(stats, numpart, numconn, heapsize, affinity)
    return (plt.errorbar(numdocs, ratios, ratiodev, None, '-+')) #, basex=2))

def plotaccess(l1stats, l2stats, l3stats, numpart, numconn, heapsize, affinity):
    (numdocs, accesstimes) = getaccesstimes(l1stats, l2stats, l3stats, numpart, numconn, heapsize, affinity)
    return (plt.semilogx(numdocs, accesstimes, '-+', basex=2))

def plotcacheaccess(l1stats, l2stats, l3stats, heapsize, plotfn):
    plots = []
    legends = []
    plots.append(plotfn(l1stats, l2stats, l3stats, 1, 1, heapsize, False))
    legends.append("1 partition, 1 connection")
    plots.append(plotfn(l1stats, l2stats, l3stats, 1, 8, heapsize, False))
    legends.append("1 partition, 8 connections")
    plots.append(plotfn(l1stats, l2stats, l3stats, 8, 1, heapsize, False))
    legends.append("8 partitions, 1 connection, no affinity")
    plots.append(plotfn(l1stats, l2stats, l3stats, 8, 1, heapsize, True))
    legends.append("8 partitions, 1 connection, with affinity")
    return (plots, legends)

def plotcache(stats, heapsize, plotfn):
    plots = []
    legends = []
    plots.append(plotfn(stats, 1, 1, heapsize, False))
    legends.append("1 partition, 1 connection")
    plots.append(plotfn(stats, 1, 8, heapsize, False))
    legends.append("1 partition, 8 connections")
    plots.append(plotfn(stats, 8, 1, heapsize, False))
    legends.append("8 partitions, 1 connection, no affinity")
    plots.append(plotfn(stats, 8, 1, heapsize, True))
    legends.append("8 partitions, 1 connection, with affinity")
    return (plots, legends)

def plotl1(stats, start, end, legendpos):
    fig = plt.figure()
    plt.xlabel("number of documents")
    plt.ylabel("L1 miss ratio")
    (plots, legends) = plotcache(stats, 1000, plotratios)
    ax = fig.get_axes()
    ax[0].set_xscale("log", basex=2)
    plt.legend(map(lambda x : x[0], plots), legends, loc=legendpos)
    plt.xlim(start, end)
    plt.ylim(0, 0.4)
    fname = ("l1_ratio_%d_%d.svg" % (start, end))
    plt.savefig(fname, format="svg")

def plotl2(stats, start, end, legendpos):
    fig = plt.figure()
    plt.xlabel("number of documents")
    plt.ylabel("L2 miss ratio")
    (plots, legends) = plotcache(stats, 1000, plotratios)
    ax = fig.get_axes()
    ax[0].set_xscale("log", basex=2)
    plt.legend(map(lambda x : x[0], plots), legends, loc=legendpos)
    plt.xlim(start, end)
    plt.ylim(0, 0.8)

    fname = ("l2_ratio_%d_%d.svg" % (start, end))
    plt.savefig(fname, format="svg")

def plotl3(stats, start, end, legendpos):
    fig = plt.figure()
    plt.xlabel("number of documents")
    plt.ylabel("L3 miss ratio")
    (plots, legends) = plotcache(stats, 1000, plotratios)
    ax = fig.get_axes()
    ax[0].set_xscale("log", basex=2)
    plt.legend(map(lambda x : x[0], plots), legends, loc=legendpos)
    plt.xlim(start, end)
    plt.ylim(0, 1.2)

    fname = ("l3_ratio_%d_%d.svg" % (start, end))
    plt.savefig(fname, format="svg")

def plotaccesstimes(l1stats, l2stats, l3stats, start, end, legendpos):
    plt.figure()
    plt.xlabel("number of documents")
    plt.ylabel("Average memory access time")
    (plots, legends) = plotcacheaccess(l1stats, l2stats, l3stats, 1000, plotaccess)
    plt.legend(map(lambda x : x[0], plots), legends, loc=legendpos)
    plt.xlim(start, end)
    plt.ylim(2, 12)
    fname = ("memory_latency_%d_%d.svg" % (start, end))
    plt.savefig(fname, format="svg")

l1stats = importAverage("cache_l1.res")
l2stats = importAverage("cache_l2.res")
l3stats = importAverage("cache_l3.res")

plotl1(l1stats, (1 << 12), (1 << 25), 4)
plotl2(l2stats, (1 << 12), (1 << 25), 2)
plotl3(l3stats, (1 << 12), (1 << 25), 2)

plotaccesstimes(l1stats, l2stats, l3stats, (1 << 12), (1 << 25), 2)
