#/usr/bin/env python
import sys
sys.path.append("../tools")
import simreport_v3
import numpy as np
import matplotlib.pyplot as plt
from operator import itemgetter
import plotparams

NUMDOCS = 40000000

def importlowdata(filename):
    avglist = simreport_v3.importAvgReports(filename)
    reportdata = []
    numpart = 0
    for report in avglist:
        if report.numhandlers == 16:
            numpart = report.numpart
            reportdata.append([report.numclients, 1.0 * (report.time_exec + report.time_queue) / 1000.0])
    return (reportdata)

def plotlowdata_client(filename):
    data = importlowdata(filename)
    numclients = []
    latencies = []
    sorteddata = sorted(data, key=itemgetter(0))
    for item in sorteddata:
        numclients.append(item[0])
        latencies.append(item[1])
    return (plt.plot(numclients, latencies, '-+'))

def importlowdata_qps(filename):
    avglist = simreport_v3.importAvgReports(filename)
    reportdata = []
    numpart = 0
    for report in avglist:
        if report.numhandlers == 16:
            numpart = report.numpart
            reportdata.append([report.numclients, report.qps])
#    sortedreport = sorted(reportdata, key = itemgetter(0))
    return (reportdata)

def plotlowdata_client_qps(filename):
    data = importlowdata_qps(filename)
    numclients = []
    qps = []
    sorteddata = sorted(data, key=itemgetter(0))
    for item in sorteddata:
        numclients.append(item[0])
        qps.append(item[1])
    return (plt.plot(numclients, qps, '-+'))

def plotclient_latency():
    plt.figure()
    plots = []
    plots.append(plotlowdata_client("bench_lowload_part1.bin"))
    plots.append(plotlowdata_client("bench_lowload_part2.bin"))
    plots.append(plotlowdata_client("bench_lowload_part4.bin"))
    plots.append(plotlowdata_client("bench_lowload_part6.bin"))
    plots.append(plotlowdata_client("bench_lowload_part8.bin"))
    plots.append(plotlowdata_client("bench_lowload_part10.bin"))
    plots.append(plotlowdata_client("bench_lowload_part12.bin"))
    legends = ["1 partition", "2 partitions", "4 partitions", "6 partitions", "8 partitions", "10 partitions", "12 partitions"]
    plt.xlabel("#clients")
    plt.ylabel("latency (ms)")
    plt.legend(map(lambda x : x[0], plots), legends, loc=2)
    plt.xlim(1, 20)
    plt.ylim(0, 50)
    plt.savefig("lowload_latency.svg", format="svg")

def plotclient_qps():
    plt.figure()
    plots = []
    plots.append(plotlowdata_client_qps("bench_lowload_part1.bin"))
    plots.append(plotlowdata_client_qps("bench_lowload_part2.bin"))
    plots.append(plotlowdata_client_qps("bench_lowload_part4.bin"))
    plots.append(plotlowdata_client_qps("bench_lowload_part6.bin"))
    plots.append(plotlowdata_client_qps("bench_lowload_part8.bin"))
    plots.append(plotlowdata_client_qps("bench_lowload_part10.bin"))
    plots.append(plotlowdata_client_qps("bench_lowload_part12.bin"))
    legends = ["1 partition", "2 partitions", "4 partitions", "6 partitions", "8 partitions", "10 partitions", "12 partitions"]
    plt.xlabel("#clients")
    plt.ylabel("qps")
    plt.legend(map(lambda x : x[0], plots), legends, loc=4)
    plt.xlim(1, 20)
    plt.savefig("lowload_qps.svg", format="svg")

plotclient_qps()
plotclient_latency()
