#/usr/bin/env python
import sys
sys.path.append("../tools")
import simreport_v2
import simreport_v3
import math
import numpy as np
import matplotlib.pyplot as plt;
from operator import itemgetter
import plotparams

def importall():
    reports = simreport_v3.importAvgReports("bench_scaling_1.bin")
    reports.extend(simreport_v3.importAvgReports("bench_scaling_2.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_3.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_4.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_5.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_6.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_7.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_8.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_9.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_10.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_11.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_12.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_noheap_1.bin"))
    return (reports)

def importaffinity_handler():
    reports = simreport_v3.importAvgReports("bench_scaling_affinity_handler_1.bin")
    return (reports)

def importaffinity():
    reports = simreport_v3.importAvgReports("bench_scaling_affinity_1.bin")
    reports.extend(simreport_v3.importAvgReports("bench_scaling_1.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_affinity_2.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_affinity_3.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_affinity_4.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_affinity_5.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_affinity_6.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_affinity_7.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_affinity_8.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_affinity_9.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_affinity_10.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_affinity_11.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_affinity_12.bin"))
    return (reports)


def find_report(reports, numpart, numhandlers, numhits):
    for report in reports:
        if (report.numpart == numpart and report.numhandlers == numhandlers and report.numhits == numhits):
            return (report)
    return None

def find_report2(reports, numpart, numhandlers, numhits, heapsize):
    for report in reports:
        if (report.numpart == numpart and report.numhandlers == numhandlers and report.numhits == numhits and report.heapsize == heapsize):
            return (report)
    return None

def import_partitions(reports, numhandlers, pstart, pend):
    numpart = [1]
    speedups = [1]
    efficiencies = [1]
    gustafson = [1]
    for part in range(pstart, pend):
        # Find baseline
        numhits = part * 2000
        baseline = find_report(reports, 1, numhandlers, numhits)
        report = find_report(reports, part, numhandlers, numhits)
        if report != None:
            speedup = (1.0 * (baseline.time_exec + baseline.time_queue) / (report.time_exec + report.time_queue))
            efficiency = (1.0 * speedup / part)
            speedups.append(speedup)
            efficiencies.append(efficiency)

            total = report.time_exec
            parallel = report.time_partitionqueue + report.time_fetcher
            handler = total - parallel
            fraction = handler * 1.0 / total
            gustafson.append(part + ((1.0 - report.numpart) * (fraction / 100.0)))
            numpart.append(part)
    return (numpart, speedups, efficiencies, gustafson)

def import_handlers(reports):
    numhandler = []
    speedups = []
    efficiencies = []
    gustafson = []
    for handler in range(1, 13):
        # Find baseline
        numhits = handler * 2000
        baseline = find_report(reports, 1, 1, numhits)
        report = find_report(reports, 1, handler, numhits)
        if baseline != None and report != None:
            speedup = (1.0 * handler * (baseline.time_exec + baseline.time_queue) / (report.time_exec + report.time_queue))
            efficiency = speedup * 1.0 / handler
            speedups.append(speedup)
            efficiencies.append(efficiency)
            total = report.time_exec
            parallel = report.time_partitionqueue + report.time_fetcher
            handlertime = total - parallel
            fraction = handlertime * 1.0 / total
 
            gustafson.append(handler + ((1.0 - report.numhandlers) * (baseline.fraction_handler / 100.0)))
            numhandler.append(handler)
    return (numhandler, speedups, efficiencies, gustafson)


def plotpart(reports, affinityreports):
    (numparts, speedups, efficiencies, gustafsons) = import_partitions(reports, 1, 2, 13)
    plt.figure()
    plots = []
    legends = []
    plots.append(plt.plot(numparts, speedups, '-+'))
    legends.append("No affinity")
    plots.append(plt.plot(numparts, gustafsons, '-+'))
    legends.append("Gustafsons")
    plots.append(plt.plot(numparts, numparts))
    legends.append("Linear")
    (numparts, speedups, efficiencies, gustafsons) = import_partitions(affinityreports, 1, 2, 13)
    plots.append(plt.plot(numparts, speedups, '-+'))
    legends.append("Affinity")

    plt.legend(map(lambda x : x[0], plots), legends, loc=2)
    plt.ylim(1, 12)
    plt.xlim(1, 12)
    plt.savefig("scalability_partitions_speedup.svg", format="svg")

def plothandler(reports, affinityreports):
    (numparts, speedups, efficiencies, gustafsons) = import_handlers(reports)
    plt.figure()
    splot = plt.plot(numparts, speedups, '-+')
    gplot = plt.plot(numparts, gustafsons, '-+')
    (numparts, speedups, efficiencies, gustafsons) = import_handlers(affinityreports)
    splot_affinity = plt.plot(numparts, speedups, '-+')
    plt.legend((splot[0], gplot[0], splot_affinity[0]), ('Vsim no affinity', 'Gustafson', 'Vsim with affinity'), loc=2)

    plt.ylim(1, 12)
    plt.xlim(1, 12)
    plt.savefig("scalability_handlers_speedup.svg", format="svg")

def autolabel(rects):
    for rect in rects:
        height = rect.get_height()
        plt.text(rect.get_x()+rect.get_width()/2., 0.05 + height, '%.1f'%float(height), ha='center', va='bottom')

def plotcompare(reports):
    numhandler = []
    numhits = []
    speedups = []

    baseline = find_report2(reports, 1, 1, 16000, 1000)
    report = find_report2(reports, 1, 8, 16000, 1000)
    speedup = [8]
    speedup.append((report.qps_mean * 1.0 / baseline.qps_mean))
    baseline = find_report2(reports, 1, 1, 16000, 1000)
    report = find_report2(reports, 1, 8, 16000, 16000)
    speedup.append((report.qps_mean * 1.0 / baseline.qps_mean))
    plt.figure()
    b3 = plt.bar(0.25, speedup[0], 0.5, color='g')
    b1 = plt.bar(1.25, speedup[1], 0.5, color='r')
    b2 = plt.bar(2.25, speedup[2], 0.5)
    plt.xlim(0, 3)
    plt.xticks([0.5, 1.5, 2.5], ('Ideal', 'Heapsize = 1000', 'Heapsize = 16000'))
    autolabel(b1)
    autolabel(b2)
    autolabel(b3)
    plt.savefig("scalability_handlers_speedup_compare.svg", format="svg")
    return

def plotaffinity(reports):
    (numparts, speedups, efficiencies, gustafsons) = import_partitions(reports, 1, 5, 9)
    plt.figure()
    linear = plt.plot(numparts, numparts, '-+')
    aff = plt.plot(numparts, speedups, '-+')

reports = importall()
affinityreports = importaffinity()
affinityreports_handler = importaffinity_handler()
plotpart(reports, affinityreports)
plothandler(reports, affinityreports_handler)
