#/usr/bin/env python
import sys
sys.path.append("../tools")
import numpy as np
import matplotlib.pyplot as plt;
import simreport_v3
import plotparams


def importreports():
    reports = simreport_v3.importAvgReports("bench_scaling_fixed_1.bin")
    reports.extend(simreport_v3.importAvgReports("bench_scaling_fixed_2.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_fixed_3.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_fixed_4.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_fixed_5.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_fixed_6.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_fixed_7.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_fixed_8.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_fixed_9.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_fixed_10.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_fixed_11.bin"))
    reports.extend(simreport_v3.importAvgReports("bench_scaling_fixed_12.bin"))
    return (reports)

def plotfractions(avglist):
    fracs_qh = []
    fracs_pq = []
    fracs_part = []
    fracs_fetch = []
    N = 10
    for report in avglist:
        if report.numpart > N:
            continue
        totaltime = report.time_exec
        report.time_handler = report.time_exec - (report.time_fetcher + report.time_partitionqueue)
        frac_qh =  report.time_handler * 1.0 / totaltime
        frac_fetch = report.time_fetcher * 1.0 / totaltime
        frac_pq = report.time_partitionqueue * 1.0 / totaltime
        fracs_qh.append(frac_qh)
        fracs_pq.append(frac_pq)
        fracs_part.append(frac_qh + frac_pq)
        fracs_fetch.append(frac_fetch)

    ind = np.arange(N) + 0.6
    width = 0.8

    qhplot = plt.bar(ind, fracs_qh, width, color='b')
    pqplot = plt.bar(ind, fracs_pq, width, color='r', bottom=fracs_qh)
    fplot = plt.bar(ind, fracs_fetch, width, color='g', bottom=fracs_part)
    plt.legend((qhplot[0], pqplot[0], fplot[0]), ('QueryHandler', 'PartitionQueue', 'Fetcher'), loc=1)
    plt.xticks(ind + (width / 2.0), map(str, (x for x in range(1, 13))))
    plt.ylabel("Fractions")
    plt.xlabel("Partitions")
    plt.ylim(0, 1.0)
    plt.xlim(0, 11)
    plt.savefig("scalability_part_fractions.svg", format="svg")
    plt.ylim(0, 0.1)
    plt.legend((qhplot[0], pqplot[0], fplot[0]), ('QueryHandler', 'PartitionQueue', 'Fetcher'), loc=2)
    plt.savefig("scalability_part_fractions_zoom.svg", format="svg")

def plotamdahl(avglist, size):
    baseline = None
    actualdata = []
    for report in avglist:
        if report.numpart == 1 and report.numhandlers == 1:
            baseline = report
            break

    basefrac = (report.time_handler * 1.0 / report.time_exec)
    numpart = []
    amdahldata = []
    gustafsondata = []
    for i in range(1, size):
        numpart.append(i)
        amdahldata.append(1.0 / (basefrac + ((1.0 - basefrac) / (1.0 * i))))
        gustafsondata.append(i + ((1.0 - i) * basefrac))
    plt.figure()
    plots = []
    legends = []
    plots.append(plt.plot(numpart, amdahldata))
    legends.append("Amdahl")
    plots.append(plt.plot(numpart, gustafsondata))
    legends.append("Gustafson")
    plots.append(plt.plot(numpart, numpart))
    legends.append("Linear")
    plt.xlabel("# partitions")
    plt.ylabel("speedup")
    plt.xlim(1, size)
    plt.legend(map(lambda x : x[0], plots), legends, loc=2)
    plt.savefig("scalability_part_fractions_amdahl.svg", format="svg")
   
results = importreports()
plotamdahl(results, 256)
plotfractions(results)
