#include <iostream>
#include <sys/time.h>
#include "task.h"
#include "fetcher.h"
#include "queryhandler.h"
#include "misc.h"

namespace vsim {

Fetcher::Fetcher(const VSimStore &ds, PartitionQueue &queue,
    docid_t id, unsigned long seed, unsigned int hid, docid_t maxdocid) :
    _ds(ds),
    _queue(queue),
    _id(id),
    _handlerid(hid),
    _generator(seed),
    _dist(0, maxdocid - 1),
    _vgen(_generator, _dist),
    _done(2)
{
}

Fetcher::~Fetcher()
{
}

void
Fetcher::run(void)
{
    bool stop = false;
    while (1) {
        Query::SP qsp = _queue.dequeue(stop);
        if (stop)
            break;
        HitCollector &hits(qsp->getCollector(_id));
        unsigned long totalhits = 0;

        const PartitionRange &slots(qsp->getRange(_id));
        // Generate a hit for each slot we have in our partition
        for (docid_t s = 0; s < slots.size(); s++) {
            const Slot &slot(slots[s]);
            unsigned long numhits = qsp->getHits(slot);
            totalhits += numhits;
            for (unsigned long i = 0; i < numhits; i++) {
                docid_t doc = (_vgen() % (slot.second - slot.first)) + slot.first;
                hits.addHit(doc, _ds.getRank(doc));
            }
        }
        hits.sortBuffer();
        qsp->notify(_id, totalhits);
    }
    _done.wait();
}

void
Fetcher::shutdown(void)
{
    _queue.shutdown();
    _done.wait();
}

bool
Fetcher::getPreferredCPU(cpu_set_t &set)
{
#ifdef AFFINITY
    CPU_ZERO(&set);
    CPU_SET(_id % NUMCPU, &set);
    return (true);
#else
    return false;
#endif
}

} // end vsim
