#ifndef _VSIM_FETCHER_H_
#define _VSIM_FETCHER_H_
#include <boost/shared_ptr.hpp>
#include <boost/random/linear_congruential.hpp>
#include <boost/random/uniform_int.hpp>
#include <boost/random/variate_generator.hpp>
#include "task.h"
#include "query.h"
#include "vsimstore.h"
#include "cond.h"
#include "barrier.h"
#include "partitionqueue.h"

namespace vsim {

typedef boost::minstd_rand base_generator_type;

/**
 * The Fetcher class is in charge of generating document ids according to the
 * number of hits given and retrieve the document ids and their rank from the
 * document store. Finally, the query is notified when the fetcher is finished.
 */
class Fetcher : public Task {
public:
    typedef boost::shared_ptr<Fetcher> SP;
    /**
     * Construct a Fetcher, initialize the range in which valid document ids can be
     * constructed.
     */
    Fetcher(const VSimStore &, PartitionQueue &, partid_t, unsigned long, unsigned int, docid_t);
    virtual ~Fetcher();

    /**
     * Generate docids to be fetched from the document store. Retrieve these
     * documents and merge them with documents from the other threads.
     */
    void run(void);

    /**
     * Shutdown Fetcher. Signal the PartitionQueue to wake up all other fetchers
     * and exit.
     */
    void shutdown(void);

    /**
     * Return our affinity preference.
     */
    bool getPreferredCPU(cpu_set_t &);

private:
    const VSimStore &_ds;
    PartitionQueue &_queue;
    partid_t _id;
    unsigned int _handlerid;
    base_generator_type _generator;
    boost::uniform_int <> _dist;
    boost::variate_generator<base_generator_type&, boost::uniform_int<> > _vgen;
    Barrier _done;
};

} // end vsim

#endif
