#include <algorithm>
#include <iostream>
#include "hitcollector.h"
#include "hitvector.h"

namespace vsim {

HitCollector::HitCollector(docid_t maxhits) :
    _hitvector(maxhits),
    _maxhits(maxhits),
    _state(COLLECTHIT)
{
}

void
HitCollector::considerForHit(docid_t docId, rank_t rank)
{
    if (_hitvector.empty()) return;

    if (rank > _hitvector[0].second) {
        std::pop_heap(_hitvector.begin(), _hitvector.end(), ScoreComparator());
        _hitvector.back().first = docId;
        _hitvector.back().second = rank;
        std::push_heap(_hitvector.begin(), _hitvector.end(), ScoreComparator());
    }
}

void
HitCollector::addHit(docid_t docid, rank_t rank)
{
    switch (_state) {
    case COLLECTHIT:
        // Add new hits until we reach the limit
        if (_hitvector.size() < _maxhits) {
            _hitvector.add(Hit(docid, rank));
        } else {
            _state = REPLACEHIT;
            std::make_heap(_hitvector.begin(), _hitvector.end(), ScoreComparator());
            considerForHit(docid, rank);
        }
        break;
    case REPLACEHIT:
        considerForHit(docid, rank);
        break;
    }
}

void
HitCollector::sortBuffer(void)
{
    _hitvector.sort();
}

void
HitCollector::printHits(void) const
{
    std::cout << "Hits: \n";
    for (unsigned int i = 0; i < _hitvector.size(); i++) {
        std::cout << "\t" << _hitvector[i].first << " " << _hitvector[i].second << std::endl;
    }
}

} // end vsim
