#ifndef _VSIM_HITCOLLECTOR_H_
#define _VSIM_HITCOLLECTOR_H_
#include <vector>
#include <stdint.h>
#include "hitvector.h"
#include "types.h"

namespace vsim {

enum HitState { COLLECTHIT, REPLACEHIT };

/**
 * The HitCollector class can be instructed to store hits in a fashion similar
 * to the same class in Vespa.
 */
class HitCollector
{
public:
    /**
     * Create a HitCollector with a internal buffer of the size of the argument.
     */
    HitCollector(docid_t);

    /**
     * Add a new hit to the hitcollector heap using id and rank from document.
     */
    void addHit(docid_t, rank_t);

    inline unsigned long size(void) const { return _hitvector.size(); }
    inline const HitVector &getResultBuffer(void) const { return _hitvector; }

    /**
     * Reset everything to the initial state.
     */
    void reset(void);

    /**
     * Sort the internal buffer.
     */
    void sortBuffer(void);

    /**
     * Organize after rank and document id.
     */
    class ScoreComparator {
    public:
        ScoreComparator() {}
        bool operator() (const Hit & lhs, const Hit & rhs) const {
            if (lhs.second == rhs.second) {
                return lhs.first < rhs.first;
            }
            return lhs.second >= rhs.second; // oparator for min-heap
        }
    };
    void printHits(void) const;

private:
    /**
     * Exchange hit on the heap if the provided hit is higher ranked than the
     * lowest ranked in the heap.
     */
    void considerForHit(docid_t, rank_t);

    HitVector _hitvector;
    docid_t _maxhits;
    HitState _state;
};

} // end vsim

#endif
