#include <iostream>
#include "hitvector.h"
#include "searchlib/common/sort.h"

namespace vsim {

void
HitVector::sort(void)
{
    search::ShiftBasedRadixSorter<Hit, ValueRadix, ValueCompare, 56>::radix_sort(ValueRadix(), ValueCompare(), &_buffer[0], _len);
}

void
HitVector::merge(const HitVector &op1, const HitVector &op2)
{
    docid_t i_op1 = 0;
    docid_t i_op2 = 0;

    while (i_op1 < op1.size() &&
           i_op2 < op2.size() && 
           _len < _buffer.size()) {
        const Hit &h1(op1[i_op1]);
        const Hit &h2(op2[i_op2]);
        // If left operand is smaller than right
        if (h1.second > h2.second) {
            add(h1);
            i_op1++;
        } else if (h1.second < h2.second) {
            add(h2);
            i_op2++;
        } else {
            add(h1);
            i_op1++;
            if (_len < _buffer.size()) {
                add(h2);
                i_op2++;
            }
        }
    }
    if (_len < _buffer.size()) {
        // According to the condition above, only one of these while loops will
        // get executed
        while (i_op1 < op1.size() &&
               _len < _buffer.size()) {
            add(op1[i_op1++]);
        }
        while (i_op2 < op2.size() &&
               _len < _buffer.size()) {
            add(op2[i_op2++]);
        }
    }
}

void
HitVector::printHits(void) const
{
    for (unsigned int i = 0; i < _len; i++) {
        std::cout << "\t" << _buffer[i].first << " " << _buffer[i].second << std::endl;
    }
}

} // end vsim
