#ifndef _VSIM_MERGEQUEUE_H_
#define _VSIM_MERGEQUEUE_H_
#include "queue.h"
#include "types.h"

namespace vsim {

/**
 * A merge queue is a queue of partition ids that are ready for merging into the
 * result buffer.
 */
class MergeQueue : public Queue<partid_t>
{
public:
    /**
     * Enqueue a partition identifier ready for merging
     */
    void enqueue(partid_t id) {
        _qlock.lock();
        _queue.push_back(id);
        _qlock.notify();
        _qlock.unlock();
    }
    /**
     * Dequeue a partition identifier ready for merging. If the queue is empty,
     * block the caller until it is not empty.
     */
    partid_t dequeue(void) {
        _qlock.lock();
        while (_queue.size() == 0) {
            _qlock.wait();
        }
        partid_t ret = _queue.front();
        _queue.pop_front();
        _qlock.unlock();
        return ret;
    }
};

} // end vsim

#endif
