#ifndef _VSIM_PARTITIONQUEUE_H_
#define _VSIM_PARTITIONQUEUE_H_
#include "queue.h"
#include "types.h"

namespace vsim {

/**
 * A partition queue implements a queue of queries as well as the possibility of
 * timestamping insertion and removal from the queue.
 */
class PartitionQueue : public Queue<Query::SP>
{
public:
    PartitionQueue(partid_t id) : _id(id) {}

    /**
     * Enqueue a query into the partition queue, and record its insertion point
     * for the particular partition.
     */
    void enqueue(const Query::SP &qsp) {
        _qlock.lock();
        _queue.push_back(qsp);
        qsp->insertPartitionQueue(_id);
        _qlock.notify();
        _qlock.unlock();
    }

    /**
     * Dequeue a query from the partition queue, and record it removal point for
     * the partition.
     */
    Query::SP dequeue(bool &stop) {
        _qlock.lock();
        while (_queue.size() == 0) {
            if (_shutdown) {
                stop = true;
                _qlock.unlock();
                return Query::SP();
            }
            _qlock.wait();
        }
        Query::SP ret = _queue.front();
        _queue.pop_front();
        ret->removePartitionQueue(_id);
        _qlock.notifyAll();
        _qlock.unlock();
        return ret;
    }
private:
    partid_t _id;
};

} // end vsim

#endif
