#ifndef _VSIM_QUERYHANDLER_H_
#define _VSIM_QUERYHANDLER_H_
#include <boost/shared_ptr.hpp>
#include <sched.h>
#include <stdint.h>
#include "task.h"
#include "query.h"
#include "fetcher.h"
#include "misc.h"
#include "vsimstat.h"
#include "barrier.h"
#include "queryqueue.h"
#include "partitionqueue.h"

namespace vsim {

/**
 * A queryhandler is responsible for synchronizing document ranking and sorting
 * of the result.
 */
class QueryHandler : public Task
{
public:
    QueryHandler(std::vector<PartitionQueue> &, VSimStat &, QueryQueue &);
    ~QueryHandler();

    /**
     * Shutdown QueryHandler. Wake up all handlers waiting on the same
     * queue and exit.
     */
    void shutdown(void);

    /**
     * The main loop of a QueryHandler thread. Listens on the query queue for
     * new queries and dispatches these queries to all fetchers. Immediately
     * starts merging of completed results.
     */
    void run(void);

    /**
     * The QueryHandler should be able to run on any CPU.
     */
    bool getPreferredCPU(cpu_set_t &set) { return false; }

private:
    std::vector<PartitionQueue> &_partitions;
    QueryQueue &_queue;
    VSimStat &_st;
    Barrier _done;
};

} // end vsim

#endif
