#ifndef _VSIM_QUERYQUEUE_H_
#define _VSIM_QUERYQUEUE_H_
#include "queue.h"
#include "query.h"

namespace vsim {

/**
 * A QueryQueue implements a queue of queries with the ability to timestamp
 * insertion and removal from the queue. Only one instance of this queue should
 * exist.
 */
class QueryQueue : public Queue<Query::SP>
{
public:
    QueryQueue(unsigned int maxsize) : Queue<Query::SP>(maxsize) {}

    /**
     * Enqueue a query and record the time of insertion. If the queue is full,
     * block the caller until it is not.
     */
    void enqueue(const Query::SP &qsp) {
        _qlock.lock();
        while (_queue.size() >= _maxsize) {
            _qlock.wait();
        }
        _queue.push_back(qsp);
        qsp->insertQueryQueue();
        _qlock.notify();
        _qlock.unlock();
    }

    /**
     * Dequeue the next query from the queue. If the queue is empty, block the
     * caller until it is not.
     */
    Query::SP dequeue(bool &stop) {
        _qlock.lock();
        while (_queue.size() == 0) {
            if (_shutdown) {
                stop = true;
                _qlock.unlock();
                return Query::SP();
            }
            _qlock.wait();
        }
        Query::SP ret = _queue.front();
        _queue.pop_front();
        ret->removeQueryQueue();
        _qlock.notifyAll();
        _qlock.unlock();
        return ret;
    }
};

} // end vsim

#endif
