#ifndef _VSIM_QUEUE_H_
#define _VSIM_QUEUE_H_
#include <deque>
#include "cond.h"

namespace vsim {

/**
 * The Queue template supports a queue of various size and implements
 * wait and notification mechanism on shutdown.
 */
template <typename T>
class Queue {
public:
    Queue(unsigned int maxsize) : _maxsize(maxsize), _shutdown(false) {}
    Queue() : _maxsize(0), _shutdown(false) {}
    virtual ~Queue() {}
    /**
     * Block the caller until all elements in the queue are dequeued.
     */
    void waitUntilEmpty(void) {
        _qlock.lock();
        while (_queue.size() > 0) {
            _qlock.wait();
        }
        _qlock.unlock();
    }

    /**
     * Notify all listeners that the queue will be destroyed.
     */
    void shutdown(void) {
        _qlock.lock();
        _shutdown = true;
        _qlock.notifyAll();
        _qlock.unlock();
    }
protected:
    std::deque<T> _queue;
    Cond _qlock;
    unsigned int _maxsize;
    bool _shutdown;
};

} // end vsim

#endif
