#ifndef _VSIM_RESULTBUFFER_H_
#define _VSIM_RESULTBUFFER_H_
#include <vector>
#include <boost/shared_ptr.hpp>
#include <stdint.h>
#include "hitcollector.h"
#include "types.h"

namespace vsim {

/**
 * A ResultBuffer contains result buffers for all partitions, and a result
 * partition to which the buffers of all partitions may be merged.
 */
class ResultBuffer
{
public:
    ResultBuffer(docid_t numhits, partid_t numpart) :
        _collectors(numpart, numhits),
        _result(2, numhits),
        _resultptr(0)
    { }
        
    inline HitCollector &getHandle(partid_t partid) { return _collectors[partid]; }
    /**
     * Merge the hits from partition with identifier specified as argument.
     */
    void mergeHits(partid_t);
    void printHits(void) const;

private:
    std::vector<HitCollector> _collectors;
    std::vector<HitVector> _result;
    unsigned int _resultptr;
};

} // end vsim

#endif
