#include <iostream>
#include <errno.h>
#include <string.h>
#include "thread.h"
#include "threadpool.h"
#include "task.h"

namespace vsim {

Thread::Thread() :
    _finished(false),
    _busy(false),
    _started(false)
{
}

Thread::~Thread()
{
}

void
Thread::join(void)
{
    _dispatched.lock();
    _finished = true;
    _dispatched.notify();
    _dispatched.unlock();
    if (_started)
        pthread_join(_handle, NULL);
}

bool
Thread::dispatch(Task::SP t)
{
    _dispatched.lock();
    if (_busy) {
        _dispatched.unlock();
        return (false);
    }
    // First time this thread was picked
    if (!_started) {
        _started = true;
        pthread_create(&_handle, NULL, Thread::trampoline, this);
    }
    _task = t;
    _busy = true;
    _dispatched.notify();
    _dispatched.unlock();
    return (true);
}

void
Thread::run(void)
{
    _tid = (pid_t)syscall(186); // gettid x86-64
    if (sched_getaffinity(_tid, sizeof(_set), &_set) != 0) {
        std::cout << "Error getting initial affinity of thread " << _tid << ": "
            << strerror(errno) << std::endl;
    }
    _dispatched.lock();
    while (!_finished) {
        while (!_busy) {
            if (_finished) {
                break;
            }
            _dispatched.wait();
        }
        _dispatched.unlock();
        if (_finished)
            break;
        cpu_set_t tset;
        if (_task->getPreferredCPU(tset)) {
            // Set preferred affinity of the running task
            if (sched_setaffinity(_tid, sizeof(tset), &tset) != 0)
                std::cout << "Error setting affinity of thread " << _tid << std::endl;
        } else {
            // Reset affinity settings of this thread
            if (sched_setaffinity(_tid, sizeof(_set), &_set) != 0)
                std::cout << "Error setting affinity of thread " << _tid << std::endl;
        }
        _task->run();
        _dispatched.lock();
        _busy = false;
        _task.reset();
    }
}

void *
Thread::trampoline(void *arg)
{
    Thread *t = (Thread *)arg;
    t->run();
    return (NULL);
}

} // end vsim
