#ifndef _VSIM_THREAD_H_
#define _VSIM_THREAD_H_
#include <pthread.h>
#include "task.h"
#include "cond.h"

namespace vsim {

class ThreadPool;

/**
 * A Thread is the internal wrappers around POSIX threads used by the threadpool
 * to dispatch the runnable entities implementing the Task interface.
 */
class Thread {
public:
    typedef boost::shared_ptr<Thread> SP;

    Thread();
    ~Thread();
    void join(void);

    /**
     * Dispatch a task to this thread. If the thread is busy with another task and
     * the task not started, return false.
     */
    bool dispatch(Task::SP);
    void run(void);
    static void *trampoline(void *);
    friend class ThreadPool;
private:
    pthread_t _handle;
    Cond _dispatched;
    bool _finished;
    bool _busy;
    bool _started;
    pid_t _tid;
    cpu_set_t _set;
    Task::SP _task;
};

} // end vsim

#endif
