#ifndef _VSIM_VSIM_H_
#define _VSIM_VSIM_H_
#include <string>
#include "vsimreport.h"
#include "vsimstat.h"
#include "vsimconfig.h"
#include "vsimstore.h"
#include "querygenerator.h"

namespace vsim {

/**
 * The VSim class implements the simulator run-time. The class may generate a
 * report of the simulator run.
 */
class VSim {
public:
    VSim(VSimConfig &vcfg) : _vcfg(vcfg) { }
    ~VSim() { }

    /**
     * Dispatch a set of threads (equal to the number of partitions) to seed the
     * data storage in order to get memory optimally allocated to each CPUs DRAM
     * controller.
     */
    void seed(VSimStore &);
    QueryGenerator::SP createQueryGenerator(QueryQueue &,
        const PartitionSpecSP &, unsigned int);

    void run(const std::string &);
    VSimReport doRun(VSimStore &);
private:
    VSimConfig &_vcfg;
};

} // end vsim

#endif
