#include <iostream>
#include <unistd.h>
#include "vsimconfig.h"
#include "vsimstore.h"
#include "querygenerator.h"
#include "misc.h"

#include <boost/math/distributions/normal.hpp>
#include <boost/math/distributions/poisson.hpp>
#include <boost/math/distributions/negative_binomial.hpp>

namespace vsim {

using boost::math::pdf;
using boost::math::cdf;

#define calcProb(s, start, end) (cdf((s), (end)) - cdf((s), (start)))


PartitionSpecSP
VSimConfig::createPartitionSpec(void)
{
    PartitionSpecSP ps(new PartitionSpec);

    docid_t maxdocid = getMaxDocId();
    partid_t numpart = getNumPartitions();
    docid_t numslots = getNumSlots();
    docid_t num_per_part = (maxdocid / numpart);
    docid_t num_per_slot = (num_per_part / numslots);

    double totalprob = 0.0;
    partid_t i = 0;
    while (maxdocid > 0) {
        docid_t start = i * num_per_part;
        docid_t range = (num_per_part >= maxdocid) ? maxdocid : num_per_part;
        if (i == (numpart - 1))
            range = maxdocid;
        PartitionRange pr;
        docid_t slotid = range;
        docid_t slotnum = 0;
        while (slotid > 0) {
            docid_t slotstart = (slotnum * num_per_slot) + start;
            docid_t slotrange = (num_per_slot >= slotid) ? slotid : num_per_slot;
            if (slotnum == (numslots - 1))
                slotrange = slotid;
            docid_t slotend = slotstart + slotrange;
            // TODO: Support different 
            double prob = 0.0;
            switch (_slotdist) {
            default: // Fallthrough, default is BINOMIAL
            case BINOMIAL:
                prob = (double)slotrange/(double)getMaxDocId();
                break;
            case GAUSS:
                {
                    boost::math::normal gauss(getMaxDocId() / 2, getMaxDocId() / 4);
                    prob = calcProb(gauss, slotstart, slotend);
                }
                break;
            case POISSON:
                {
                    boost::math::poisson poisdist(getMaxDocId() / 2);
                    prob = calcProb(poisdist, slotstart, slotend);
                }
                break;
            case GEOMETRIC:
                {
                    // First parameter gives average for geometric, and second
                    // gives steepness of the curve
                    boost::math::negative_binomial negbinom(1, 0.99);
                    prob = calcProb(negbinom, ((double)slotstart / (double)getMaxDocId()), ((double)slotend / (double)getMaxDocId()));
                }
            }
            totalprob += prob;
            pr.push_back(Slot(slotstart, slotend, prob));
            slotid -= slotrange;
            slotnum++;
        }
        (*ps).push_back(pr);
        maxdocid -= range;
        i++;
    }
    // Spread the rest of the probabilities on the rest of the slots according
    // to the ration compared to totalprob!
    double remaining = 1.0 - totalprob;
    double newtotal = 0.0;
    for (docid_t i = 0; i < (*ps).size(); i++) {
        for (docid_t j = 0; j < (*ps)[i].size(); j++) {
            double prob = (*ps)[i][j].prob;
            double ratio = prob / totalprob;
            (*ps)[i][j].prob += (remaining * ratio);
            newtotal += (*ps)[i][j].prob;
        }
    }
    return (ps);
}

} // end vsim
