#ifndef _VSIM_VSIMCONFIG_H_
#define _VSIM_VSIMCONFIG_H_
#include <string>
#include "types.h"

namespace vsim {

/**
 * The VSimConfig class handles configuration for the simulator run time. It can
 * be used to inspect the configuration status from the system.
 */
class VSimConfig
{
public:
    VSimConfig(docid_t numpart = 1,
               docid_t numslots = 1,
               unsigned long maxconn = 1,
               unsigned long clients = 1,
               docid_t maxdocid = 10000,
               docid_t numhits = 1000,
               docid_t hitheapsize = 100,
               unsigned long numattr = 0,
               AttributeIndexVector attrrange = AttributeIndexVector(0),
               SlotDistribution slotdist = BINOMIAL,
               unsigned long simtime = 10,
               unsigned long numrounds = 1,
               const std::string &qrlog = "") :
        _numpart(numpart),
        _numslots(numslots),
        _maxconn(maxconn),
        _clients(clients),
        _maxdocid(maxdocid),
        _numhits(numhits),
        _hitheapsize(hitheapsize),
        _numattr(numattr),
        _attrindices(attrrange),
        _slotdist(slotdist),
        _simtime(simtime),
        _numrounds(numrounds),
        _qrlog(qrlog)
    {
    }

    docid_t getNumPartitions(void) const { return _numpart; }
    docid_t getNumSlots(void) const { return _numslots; }
    unsigned int getMaxConnections(void) const { return _maxconn; }
    unsigned long getNumClients(void) const { return _clients; }
    docid_t getMaxDocId(void) const { return _maxdocid; }
    docid_t getNumHits(void) const { return _numhits; }
    docid_t getHitHeapSize(void) const { return _hitheapsize; }
    SlotDistribution getSlotDistribution(void) const { return _slotdist; }
    const std::string &getQRSLogFile(void) const { return _qrlog; }
    PartitionSpecSP createPartitionSpec(void);
    void exportConfig(const std::string &);
    unsigned long getSimTime(void) const { return _simtime; }
    unsigned long getNumberOfRounds(void) const { return _numrounds; }
    unsigned int getNumAttributes(void) const { return _numattr; }
    const AttributeIndexVector &getAttributeIndices(void) const { return _attrindices; }
    
private:
    docid_t _numpart;
    docid_t _numslots;
    unsigned int _maxconn;
    unsigned long _clients;
    docid_t _maxdocid;
    docid_t _numhits;
    docid_t _hitheapsize;
    unsigned int _numattr;
    AttributeIndexVector _attrindices;
    SlotDistribution _slotdist;
    unsigned long _simtime; 
    unsigned long _numrounds;
    std::string _qrlog;
};

} // end vsim

#endif
