#ifndef _VSIM_VSIMREPORT_H_
#define _VSIM_VSIMREPORT_H_
#include <vector>
#include <string>
#include <fstream>
#include <stdint.h>
#include "types.h"

namespace vsim {

/**
 * The report class describes a simulation run. The report may be
 * exported to file, or imported from a file.
 */
class VSimReport
{
public:
    void exportReport(const char *);
    void importReport(const char *);
    int importReportData(std::ifstream &);
    void printReport(void);
    static SlotDistribution distIntToName(uint64_t);
    static uint64_t distNameToInt(SlotDistribution);

    uint64_t numpart;               // Number of partitions
    uint64_t numhandlers;           // Number of query handlers
    uint64_t numhits;               // Number of hits per query (if not using qrlog)
    uint64_t numclients;            // Number of clients
    uint64_t heapsize;              // Number of hits returned
    uint64_t numdocs;               // Number of documents stored
    uint64_t numattr;               // Number of attributes per document
    uint64_t numslots;              // Number of slots per partition
    uint64_t distribution;          // Which probability distribution used
    uint64_t numqueries;            // Number of queries finished
    uint64_t simtime;               // Simulation time, in seconds
    uint64_t cputime;               // CPU time, in seconds
    uint64_t qps;                   // Queries per simulation second
    uint64_t qps_weighted;          // Queries per CPU second
    uint64_t time_queue;            // Time spent in query queue
    uint64_t time_exec;             // Time spent on query execution
    uint64_t time_fetcher;          // Time spent in fetcher
    uint64_t time_partitionqueue;   // Time spent in partition queue
    uint64_t time_handler;          // Time spent in query handler
    double fraction_handler;        // Fraction of execution time spent in handler
    HitStat hitstats;               // Per partition hit counts
    std::vector<uint64_t> partitionqueuetimes; // Per partition queue times
    std::vector<uint64_t> fetchertimes;        // Per partition fetcher execution times
    uint64_t numindices;                       // Number of attribute indices
    AttributeIndexVector attributeindices;     // Attributes used for calculating rank
    std::vector<uint64_t> partitionscheme;     // Number of slots per partition at the end
};

std::vector<VSimReport> importReportBatch(const char *);

} // end vsim

#endif
