#ifndef _VSIM_VSIMSTAT_H_
#define _VSIM_VSIMSTAT_H_
#include <sys/time.h>
#include <sys/resource.h>
#include <stdint.h>
#include "vsimconfig.h"
#include "vsimreport.h"
#include "vsimtime.h"
#include "mutex.h"
#include "cond.h"
#include "types.h"
#include "misc.h"

namespace vsim {

/**
 * The VSimStat class keeps statistics. The statistics object can be used by the
 * internal query classes to report relevant information. These results can be
 * used to create a report which can be stored to disk.
 * 
 * The class may also generate a new partition specification from the hit
 * statistics of each partition in order to handle dynamic workloads.
 */
class VSimStat
{
public:
    VSimStat(VSimConfig &);
    void report(std::vector<uint64_t> &, std::vector<uint64_t> &, uint64_t, uint64_t, uint64_t);
    void startSimulation(void);
    void stopSimulation(void);
    void sleepUntilFinished(void);
    void reportHits(HitStat);
    VSimReport generateReport(const PartitionSpec &);
    PartitionSpecSP updatePartitionSpec(const PartitionSpecSP &);

private:
    VSimConfig &_vcfg;
    Mutex _hslock;
    HitStat _hs;
    HitStat _parths;
    uint64_t _numfinished;
    std::vector<uint64_t> _fetchertimes;
    std::vector<uint64_t> _partitionqueuetimes;
    uint64_t _totalmaxfetchertime;
    uint64_t _totalqueryqueuetime;
    uint64_t _totalhandlertime;
    VSimTime _startSim;
    VSimTime _stopSim;
    struct rusage _rusage_start;
    struct rusage _rusage_end;
};

} // end vsim

#endif
