#ifndef _VSIM_VSIMSTORE_H_
#define _VSIM_VSIMSTORE_H_
#include <vector>
#include "vsimconfig.h"
#include "types.h"

namespace vsim {

/**
 * A VSimStore contains a set of attributes belonging to documents. The
 * attribute arrays containing attribute data are initialized and accessed
 * through this class when asking for the rank of a document.
 */
class VSimStore
{
public:
    VSimStore(const VSimConfig &);
    ~VSimStore();

    /**
     * Make sure that the attributes are not swapped to disk.
     */
    void memlock(void);
    void memunlock(void);

    /**
     * Generate the attribute values.
     */
    void seed(unsigned long);
    void seedRange(unsigned long, docid_t, docid_t);

    /**
     * Calculate the rank of a document.
     */
    rank_t getRank(docid_t) const;

    static const unsigned long MAXRANK = 1000000;
private:
    docid_t _maxdocid;
    AttributeIndexVector _attrindex;
    AttributeVector _attributes;
};

} // end vsim
#endif
