#include <time.h>
#include <sys/time.h>
#include "vsimtime.h"

namespace vsim {

VSimTime::VSimTime()
{ }

VSimTime::VSimTime(struct timeval &val)
{
    _value.tv_sec = val.tv_sec;
    _value.tv_nsec = (val.tv_usec * 1000);
}

void
VSimTime::recordTime(void)
{
    clock_gettime(CLOCK_MONOTONIC, &_value);
}

unsigned long
VSimTime::getTimeSeconds(void)
{
    return (_value.tv_sec);
}

unsigned long
VSimTime::getTimeMilliSeconds(void)
{
    return (_value.tv_sec * 1000);
}

unsigned long
VSimTime::getTimeNanoSeconds(void)
{
    return (_value.tv_nsec);
}

unsigned long
VSimTime::getDiffMilliSeconds(VSimTime &t2)
{
    unsigned long t2sec = t2.getTimeSeconds();
    unsigned long t2nsec = t2.getTimeNanoSeconds();
    unsigned long t1sec = _value.tv_sec;
    unsigned long t1nsec = _value.tv_nsec;

    unsigned long millidiff = 0;
    if (t2sec > t1sec) {
        millidiff = (t2sec - t1sec) * MILLI_PER_SECOND;
        if (t2nsec > t1nsec) {
            millidiff += ((t2nsec - t1nsec) / NANO_PER_MILLI);
        } else {
            millidiff -= ((t1nsec - t2nsec) / NANO_PER_MILLI);
        }
    } else if (t2sec < t1sec) {
        millidiff = (t1sec - t2sec) * MILLI_PER_SECOND;
        if (t2nsec > t1nsec) {
            millidiff -= ((t2nsec - t1nsec) / NANO_PER_MILLI);
        } else {
            millidiff += ((t1nsec - t2nsec) / NANO_PER_MILLI);
        }
    } else {
        if (t2nsec > t1nsec) {
            millidiff = ((t2nsec - t1nsec) / NANO_PER_MILLI);
        } else {
            millidiff = ((t1nsec - t2nsec) / NANO_PER_MILLI);
        }
    }
    return (millidiff);
}

unsigned long
VSimTime::getDiffMicroSeconds(VSimTime &t2)
{
    unsigned long t2sec = t2.getTimeSeconds();
    unsigned long t2nsec = t2.getTimeNanoSeconds();
    unsigned long t1sec = _value.tv_sec;
    unsigned long t1nsec = _value.tv_nsec;

    unsigned long microdiff = 0;
    if (t2sec > t1sec) {
        microdiff = (t2sec - t1sec) * MICRO_PER_SECOND;
        if (t2nsec > t1nsec) {
            microdiff += ((t2nsec - t1nsec) / NANO_PER_MICRO);
        } else {
            microdiff -= ((t1nsec - t2nsec) / NANO_PER_MICRO);
        }
    } else if (t2sec < t1sec) {
        microdiff = (t1sec - t2sec) * MICRO_PER_SECOND;
        if (t2nsec > t1nsec) {
            microdiff -= ((t2nsec - t1nsec) / NANO_PER_MICRO);
        } else {
            microdiff += ((t1nsec - t2nsec) / NANO_PER_MICRO);
        }
    } else {
        if (t2nsec > t1nsec) {
            microdiff = ((t2nsec - t1nsec) / NANO_PER_MICRO);
        } else {
            microdiff = ((t1nsec - t2nsec) / NANO_PER_MICRO);
        }
    }
    return (microdiff);
}

} // end vsim
